import yj from '../../typings';
import * as React from 'react';
import * as classNames from 'classnames';
import * as BetterScroll from 'better-scroll';
import Slide, { BaseProps } from './Slide';
import Dots from './Dots';
import device from '../../utils/device';
import getDirection from '../../utils/getDirection';
import './index.scss';

export interface Props extends yj.ComponentProps, BaseProps {
  /** 单个slide宽度 */
  slideWidth?: number;
  /** 是否开启圆点 */
  dots?: boolean;
  /** 是否自动轮播 */
  autoplay?: boolean;
  /** 自动间隔时间 */
  interval?: number;
  /** 滑动后触发的事件 */
  onAfterSlide?: (index: number, direction: 'prev' | 'next', isAuto: boolean) => void;
  /** 是否开启滑动，默认为true */
  swiping?: boolean;
  /** 如果子元素会发生变化，这时需要改变refreshId */
  refreshId?: any;
}

interface State {
  /** 当前激活的索引 */
  index: number;
}

const prefixCls = 'yj-component-carousel';
const BScroll = (BetterScroll as any).default;

class Carousel extends React.Component<Props, State> {
  constructor(props: Props) {
    super(props);

    this.onBeforeSlide = this.onBeforeSlide.bind(this);
    this.onAfterSlide = this.onAfterSlide.bind(this);
  }

  public static defaultProps: Props = {
    index: 0,
    interval: 3000,
    swiping: true
  };

  public state: State = {
    index: this.props.index || 0
  };

  private slideRef!: HTMLDivElement;
  private slide!: BScroll;
  private timer: NodeJS.Timer | number | undefined;
  /** 动画执行时间 */
  private speed: number = 400;
  /** 是否是自动轮播触发的滑动 */
  private slideIsAuto: boolean = false;
  /** 总slide数量 */
  private get slideCount(): number {
    return React.Children.count(this.props.children);
  }

  public componentDidMount(): void {
    this.init();
  }

  public componentWillReceiveProps(nextProps: Props): void {
    const children = this.props.children as JSX.Element[];
    const nextChildren = nextProps.children as JSX.Element[];

    // chldren内容改变且长度变化时重新设置滚动条
    if (children !== nextChildren) {
      if (children.length !== nextChildren.length) {
        this.slide.refresh();
      }
    }

    if (this.state.index !== nextProps.index && nextProps.index !== undefined) {
      this.setState({
        index: nextProps.index
      });
      this.slide.goToPage(nextProps.index, 0);
    }

    // 更新是否可以滑动的状态
    if (this.props.swiping !== nextProps.swiping) {
      if (nextProps.swiping === true) {
        this.slide.enable();
      } else {
        this.slide.disable();
      }
    }

    // 更新是否可以自动轮播的状态
    if (this.props.autoplay !== nextProps.autoplay) {
      this.autoplay(nextProps);
    }
  }

  public componentDidUpdate(prevProps: Props): void {
    const refreshIdChange = this.props.refreshId !== prevProps.refreshId;

    if (refreshIdChange) {
      this.init();
    }
  }

  public componentWillUnmount(): void {
    this.unmount();
  }

  private init(): void {
    this.slide = new BScroll(this.slideRef, {
      scrollX: true,
      scrollY: false,
      momentum: false,
      snap: {
        loop: this.props.loop,
        threshold: 0.1,
        speed: this.speed
      },
      bounce: false,
    });

    this.slide.on('scrollEnd', this.onAfterSlide);
    this.slide.on('beforeScrollStart', this.onBeforeSlide);
    this.slide.on('touchEnd', () => {
      this.autoplay(this.props);
    });

    // 初始化自动轮播
    this.autoplay(this.props);
  }

  private unmount(): void {
    if (this.timer) {
      clearTimeout(this.timer as number);
    }

    if (this.slide) {
      this.slide.destroy();
    }
  }

  private onBeforeSlide(): void {
    if (this.props.autoplay && this.timer) {
      clearTimeout(this.timer as number);
    }
  }

  private onAfterSlide(): void {
    // 当前索引
    const nowIndex = this.slide.getCurrentPage().pageX;
    // 上一个索引，因为还没给state更新值
    const prevIndex = this.state.index;
    // 最后一个索引，因为索引是从0开始计算，总slide数量是从1开始计算，所以要-1
    const lastIndex = this.slideCount - 1;
    // slide滑动的方向，通过计算索引差值来实现
    const direction = getDirection(nowIndex, prevIndex, lastIndex);

    this.setState({
      index: nowIndex
    }, () => {
      this.autoplay(this.props);
    });

    if (this.props.onAfterSlide) {
      this.props.onAfterSlide(nowIndex, direction, this.slideIsAuto);
    }

    // 还原初始化状态
    this.slideIsAuto = false;
  }

  /** 自动轮播 */
  private autoplay(props: Props): void {
    if (props.autoplay) {
      if (this.timer) {
        clearTimeout(this.timer as number);
      }

      // 自动轮播
      this.timer = setTimeout(() => {
        this.slideIsAuto = true;
        this.slide.next();
      }, this.props.interval);
    }
  }

  public render(): JSX.Element {
    const { className, children, style, slideWidth, loop, dots, slideHeight } = this.props;
    const width = slideWidth || device.width;

    return (
      <div
        className={classNames(prefixCls, className)}
        style={{
          width,
          ...style
        }}
        ref={(el: HTMLDivElement) => this.slideRef = el}
      >
        <Slide
          className={className}
          style={style}
          slideWidth={width}
          loop={loop}
          slideCount={this.slideCount}
          slideHeight={slideHeight}
        >{children}</Slide>
        {dots ? (
          <Dots activeIndex={this.state.index} count={this.slideCount} />
        ) : null}
      </div>
    );
  }
}

export default Carousel;