import yj from '../../typings';
import * as React from 'react';
import { History } from 'history';
import { withRouter } from 'react-router-dom';
import { TransitionGroup, CSSTransition } from 'react-transition-group';
import Page from '../../app/page';
import global from '../../app/global';
import switchRouter, { Props as SwitchRouterProps } from '../../utils/switchRouter';
import historyReplace from '../../utils/historyReplace';
import pageLeave from '../../model/behavior/pageLeave';
import './index.scss';

interface BaseProps {
  routes: JSX.Element[];
}

// withRouter提供了react-router的porps
export interface Props extends BaseProps, yj.PageProps<any> {
  history: History;
}

interface State {
  /** 当前匹配的路由 */
  route: JSX.Element[];
}

// 用于第一次进入时将route的参数应用到react-router上，简单来说就是达到了一个跳转的效果
historyReplace();

class Stack extends React.Component<BaseProps> {
  constructor(props: BaseProps) {
    super(props);

    global.history = (props as Props).history;
    this.push();
  }

  public state: State = {
    route: []
  };
  /** 项目中的Page组件 */
  public static component: typeof Page[] = [];
  /** 离开页面时记录的滚动条位置 */
  public static scrollLocation: number[] = [];
  /** 是否在componentWillReceiveProps中router为空是执行了push操作 */
  private isEmptyPush: boolean = false;
  private nextProps: Props = this.props as Props;
  private transitionKey: number = 0;

  public shouldComponentUpdate(nextProps: Props): boolean {
    const props = this.props as Props;
    return nextProps.location.pathname !== props.location.pathname;
  }

  public componentWillReceiveProps(nextProps: Props): void {
    const nextHistory = nextProps.history as History;
    this.nextProps = nextProps;
    this.isEmptyPush = false;

    if (nextHistory.action === 'POP') {
      this.state.route.pop();
    } else {
      if (nextHistory.action === 'REPLACE') {
        // pop当前这个页面
        this.state.route.pop();
        this.onReplace();
        Stack.component.pop();
        Stack.scrollLocation.pop();
        // pop上一个页面
        // this.state.route.pop();
        // Stack.component.pop();
        // Stack.scrollLocation.pop();
      }

      this.push();
    }

    if (this.state.route.length === 0) {
      this.isEmptyPush = true;
      this.push();
    }
  }

  /** 从stack中获取当前页面 */
  public static getStackComponent(): typeof Page {
    return Stack.component[Stack.component.length - 1];
  }

  /** 是否是replace动作 */
  private isReplaceAction(): boolean {
    const action = this.nextProps.history.action;
    const isReplace = action === 'REPLACE';

    return isReplace;
  }

  // 对route中返回的page element进行包裹，加上transition等效果
  private getRoute(): JSX.Element {
    this.transitionKey++;

    return (
      <CSSTransition
        classNames={'yj-pagestack'}
        timeout={300}
        onEnter={this.onEnter}
        onEntered={this.onEntered}
        onExit={this.onExit}
        onExited={this.onExited}
        key={this.transitionKey}
      >
        {React.createElement(global._appRoute as () => JSX.Element, { location: this.nextProps.location })}
      </CSSTransition>
    );
  }

  private getPageStackInAppRoute(): typeof Page {
    const route = switchRouter(this.nextProps.routes, this.nextProps.location.pathname) as React.ReactElement<SwitchRouterProps>;

    if (!route) {
      console.error('路由解析失败，请检查路由映射或url是否正确');
    }

    return route.props.component;
  }

  private push(): void {
    this.state.route.push(this.getRoute());
    Stack.component.push(this.getPageStackInAppRoute());
  }

  // 获取上一个页面的类声明，因为第一次进入时不会调用Stack.page.component.push
  private getPrevPageClass(): typeof Page {
    const prevTopComponent = Stack.component[Stack.component.length - 2];

    if (prevTopComponent === undefined) {
      console.warn('页面栈数量异常，请检查Stack.page.component.push是否正确调用');
    }
    return prevTopComponent;
  }

  /** 获取页面的pageRef静态属性 */
  private getPageRootElemRef(page: typeof Page): HTMLElement | null {
    const pageRootElemRef = page._pageRootElemRef;

    if (pageRootElemRef === undefined) {
      console.warn('页面组件中pageRef不存在，请检查setPageRef是否正确调用');
    }

    return pageRootElemRef;
  }

  private onEnter = () => {
    if (this.isReplaceAction()) {
      return;
    }

    Stack.scrollLocation.push(window.scrollY);
    document.body.style.overflow = 'hidden';
  }

  private onEntered = () => {
    if (this.isReplaceAction()) {
      return;
    }

    const prevComponent = this.getPrevPageClass();
    const pageRef = this.getPageRootElemRef(prevComponent);

    // 上一个页面的离开事件
    if (prevComponent.pageLeave) {
      prevComponent.pageLeave();
    }

    if (this.isEmptyPush === false) {
      this.onPageLeave('enter');
    }

    if (pageRef) {
      pageRef.style.display = 'none';
    }

    window.scrollTo(0, 0);
    document.body.style.overflow = 'visible';
    // 因为router的pop比stack执行得要早，为了在onExite和onExited中执行的stack顺序正确
    if (this.isEmptyPush) {
      Stack.component.reverse();
      Stack.scrollLocation.reverse();
    }
  }

  private onExit = () => {
    if (this.isReplaceAction()) {
      return;
    }

    document.body.style.overflow = 'hidden';
  }

  private onExited = () => {
    if (this.isReplaceAction()) {
      return;
    }

    const prevComponent = this.getPrevPageClass();
    const pageRef = this.getPageRootElemRef(prevComponent);
    const scrollLocation = Stack.scrollLocation[Stack.scrollLocation.length - 1];

    // 上一个页面的进入事件
    if (prevComponent.pageEnter && this.isEmptyPush === false) {
      prevComponent.pageEnter();
    }

    this.onPageLeave('exit');

    if (pageRef) {
      pageRef.style.display = 'block';
    }

    document.body.style.overflow = 'visible';
    window.scrollTo(0, scrollLocation);
    Stack.component.pop();
    Stack.scrollLocation.pop();
  }

  private onReplace(): void {
    const currentComponent = Stack.component[Stack.component.length - 1];

    if (currentComponent.pageLeave) {
      currentComponent.pageLeave();
    }

    window.scrollTo(0, 0);
    document.body.style.overflow = 'visible';
  }

  private onPageLeave(type: 'enter' | 'exit'): void {
    const component = Stack.component[Stack.component.length - 2];
    const toComponent = Stack.component[Stack.component.length - 1];

    if (component && component._page && component._pageName) {
      pageLeave({
        behavior: component._page.behavior,
        type,
        now: component._pageName,
        to: toComponent._pageName || '',
        staytime: component._page._staytime
      });
    } else {
      console.warn(`页面的_page或_pageNmae静态属性为空，请检查改页面是否调用_init方法，以及是否设置_pageName`);
    }
  }

  public render(): JSX.Element[] | JSX.Element {
    return (
      <TransitionGroup>
        {this.state.route}
      </TransitionGroup>
    );
  }
}

export default withRouter(Stack as any) as typeof Stack;
