import { observable, action } from 'mobx';
import validate from '../../utils/validate';
import FormService from '../../component/Form/service';

export interface BaseProps<Value> {
  /** 提交的JSON Key值 */
  fieldKey: string;
  /** 字段显示的名称 */
  fieldName: string;
  /** 初始值 */
  defaultValue?: Value;
  /** 获取表单模型 */
  form?: () => FormService;
  /** 是否必填 */
  required?: boolean;
  /** 自定义验证函数，或者从内置函数中选择 */
  validate?: ((value: Value) => boolean) | keyof typeof validate;
  /** value改变事件 */
  onChange?: (value: Value) => void;
}

abstract class Fields<Value> {
  constructor(props: BaseProps<Value>) {

    this.fieldKey = props.fieldKey;
    this.fieldName = props.fieldName;
    this.required = props.required || false;
    this.getForm = props.form;
    this.value = props.defaultValue;
    // tslint:disable-next-line:prefer-conditional-expression
    if (typeof props.validate  === 'string') {
      this.verification = validate[props.validate] as ((value: any) => boolean);
    } else {
      this.verification = props.validate;
    }
    this.onChange = props.onChange;

    this.setValue = this.setValue.bind(this);
    this.setPrompt = this.setPrompt.bind(this);
    this.validate = this.validate.bind(this);
    this.clear = this.clear.bind(this);
    this.changeValue = this.changeValue.bind(this);
    this.getValue = this.getValue.bind(this);
    this.getPrompt = this.getPrompt.bind(this);
    this.getError = this.getError.bind(this);
  }

  /** 表单模型 */
  protected getForm: BaseProps<Value>['form'] | undefined;
  /** 验证函数的方法名 */
  private verification: ((value: Value) => boolean) | undefined ;
  /** value改变事件 */
  private onChange: ((value: Value) => void) | undefined;
  /** 字段中文名称 */
  private fieldName: string;
  /** 是否必填 */
  private required: boolean | undefined;
  /** 值 */
  @observable private value: Value | undefined;
  /** 是否输入为空 */
  @observable private empty: boolean | undefined;
  /** 是否输入错误 */
  @observable private error: boolean | undefined;
  /** 用于覆盖默认的prompt */
  @observable private prompt: string | undefined;
  /** json key值 */
  public readonly fieldKey: string;

  /** 单纯的赋值 */
  @action public setValue(value: Value | undefined): void {
    this.value = value;
  }

  /** 设置提示文字 */
  @action public setPrompt(text: string): void {
    this.prompt = text;
  }

  /** 输入验证函数 */
  @action public validate(): void {
    // 验证是否是必填
    if (this.required) {
      if (this.value) {
        this.empty = false;
      } else {
        this.empty = true;
        this.error = true;
        return;
      }
    }

    // 判断是否有自定义验证函数
    if (this.value && this.verification) {
      this.error = !this.verification(this.value);
      return;
    }

    // 判断有没有验证函数，没有的话就不需要验证
    this.error = false;
  }

  /** 清空数据状态 */
  @action public clear(): void {
    this.value = undefined;
    this.empty = false;
    this.error = false;
    this.prompt = '';
  }

  /** 对值改变时做一些联动处理 */
  @action public changeValue(value: Value): void {
    this.setValue(value);
    this.validate();

    if (this.onChange) {
      this.onChange(value);
    }

    if (this.getForm) {
      this.getForm().validate(this.fieldKey, false);
    }
  }

  public getValue(): Value | undefined {
    return this.value;
  }

  /** 提示文字 */
  public getPrompt(): string {
    if (this.prompt) {
      return this.prompt;
    } else {
      if (this.empty !== false) {
        return `${this.fieldName}必填`;
      } else if (this.error !== false) {
        return `${this.fieldName}格式不正确`;
      } else {
        return '';
      }
    }
  }

  /** 获取是否是输入错误状态 */
  public getError(): boolean | undefined {
    return this.error;
  }

  /** 获取是否必选状态 */
  public getRequired(): boolean | undefined {
    return this.required;
  }
}

export default Fields;