import stringToHex from '../../utils/stringToHex';
import isHttp from '../../utils/isHttp';
import global from '../../app/global';
const { hex_sha1 } = require('./sha1.js');

interface Options {
  ticket: string;
  appId: string;
  shareLink?: string;
  shareTitle?: string;
  shareDesc?: string;
  debug?: boolean;
}

interface OnShareParams {
  title?: string;
  desc?: string;
  imgHost?: string;
  imgUrl?: string;
  callback?: () => void;
}

export interface Share {
  /** 标题 */
  title: string;
  /** 描述 */
  desc?: string;
  /** 封面 */
  cover?: string;
}

declare const wx: any;

// const env = process.env.NODE_ENV;
const conventionMenuItem = [
  'menuItem:share:qq',
  'menuItem:share:weiboApp',
  'menuItem:share:facebook',
  'menuItem:share:QZone',
  'menuItem:editTag',
  'menuItem:delete',
  'menuItem:copyUrl',
  'menuItem:originPage',
  'menuItem:readMode',
  'menuItem:openWithQQBrowser',
  'menuItem:openWithSafari',
  'menuItem:share:email',
  'menuItem:share:brand'
];

export class WeiXin {
  private apiList: string[] = [
    'previewImage',
    'hideMenuItems',
    'onMenuShareTimeline',
    'onMenuShareAppMessage',
    'onMenuShareWeibo',
    'onMenuShareQZone',
    'showOptionMenu',
    'hideAllNonBaseMenuItem'
  ];
  public static options: Options | undefined;

  public ready(callback: () => void): void {
    if (WeiXin.options) {
      wx.ready(callback);
    }
  }

  /** 初始化配置，对应wx.config */
  public config(): void {
    if (WeiXin.options) {
      const { ticket, debug, appId } = WeiXin.options;
      const timestamp = new Date().getTime();
      const nonceStr = 'ak0os7h1dojn51ojvs5r8lfwsq3debsu';
      const url = window.location.href.split('#')[0];
      const data = `jsapi_ticket=${ticket}&noncestr=${nonceStr}&timestamp=${timestamp}&url=${url}`;
      const signature = hex_sha1(data);
      wx.config({
        debug,
        appId,
        timestamp,
        nonceStr,
        signature,
        jsApiList: this.apiList
      });
    }
  }

  /** 隐藏所有非基础按钮接口 */
  public hideAllNonBaseMenuItem(): void {
    this.ready(() => {
      wx.hideAllNonBaseMenuItem();
    });
  }

  /** 常见隐藏菜单配置 */
  public hideConventionMenuItem(): void {
    this.ready(() => {
      wx.showOptionMenu();
      wx.hideMenuItems({
        menuList: conventionMenuItem
      });
    });
  }

  /** 在常见菜单配置基础隐藏所有分享 */
  public hideShareMenuItem(): void {
    this.ready(() => {
      const menuList = [
        ...conventionMenuItem,
        'menuItem:share:appMessage',
        'menuItem:share:timeline',
        'menuItem:favorite',
      ];
      wx.showOptionMenu();
      wx.hideMenuItems({
        menuList
      });
    });
  }

  /** 预览图片 */
  public previewImage(src: string, imgHost: string): void {
    this.ready(() => {
      const uri = imgHost + src;

      if (isHttp(uri)) {
        const urls = [uri.replace('s_', 'l_')];

        wx.previewImage({
          // 当前显示图片的http链接
          current: urls[0],
          // 需要预览的图片http链接列表
          urls
        });
      }
    });
  }

  /** 微信分享 */
  public onShare(params: OnShareParams): void {
    this.ready(() => {
      const options = WeiXin.options;

      if (options && options.shareLink && options.shareTitle && options.shareDesc) {
        const { callback } = params;
        const title = params.title || options && options.shareTitle;
        const desc = params.desc || options && options.shareDesc;
        const imgUrl = params.imgUrl || 'https://testwap.cqlure.cn/assert/img/logo.jpg';
        const pathname = global.history.location.pathname;
        const link = options.shareLink + stringToHex(pathname);
        const apiList = [
          'onMenuShareTimeline',
          'onMenuShareAppMessage',
          'onMenuShareQQ',
          'onMenuShareWeibo',
          'onMenuShareQZone'
        ];

        this.hideConventionMenuItem();

        for (let i = 0; i < apiList.length; i++) {
          const api = apiList[i];
          wx[api]({
            title,
            desc,
            link,
            imgUrl,
            success: callback
          });
        }
      } else {
        console.error('微信分享失败，缺失shareLink或shareTitle或shareDesc的配置');
      }
    });
  }
}

const weixin = new WeiXin();

export default weixin;