import yj from '../../typings';
import * as React from 'react';
import * as classNames from 'classnames';
import { observer } from 'mobx-react';
import Popup from '../Popup';
import Container from '../Container';
import IconText from '../IconText';
import Choice from '../Choice';
// import { Value as ChoiceValue, CallBackOnClick as ChoiceOnClick } from '../../_base/action/Choice';
import { Data as DataItem } from '../../_base/component/Choice';
import ChoiceService from '../Choice/service';
// import FilterService from './service';
import svgRootPath from '../../utils/svgRootPath';
import './index.scss';
const upSvg = svgRootPath + require('../../assets/up.svg').id;
const downSvg = svgRootPath + require('../../assets/down.svg').id;

export interface Data {
  label: string;
  options: DataItem[];
}

export interface Props extends yj.ComponentProps, BaseItemProps {
  service?: ChoiceService[];
  /** 选项的数据 */
  data: Data[];
  /** 选中标签的样式 */
  activeLabelCls?: string;
  /** 选中选项的样式 */
  activeChoiceCls?: string;
  /** popup contentCls */
  contentMaskCls?: string;
  /** 关闭时触发 */
  onClose?: () => void;
}

interface State {
  /** 选中的label */
  selected: number | undefined;
}

interface BaseItemProps {
  /** 弹出框样式 */
  popupCls?: string;
  /** 选中图标的样式 */
  iconCls?: string;
}

const prefixCls = 'yj-component-filter';

@observer class Filter extends React.Component<Props, State> {
  constructor(props: Props, state: State) {
    super(props, state);

    this.setSelected = this.setSelected.bind(this);
    this.onClose = this.onClose.bind(this);
  }

  public state: State = {
    selected: undefined
  };
  private elemRef: HTMLDivElement | undefined;

  private saveElemRef = (el: Container) => {
    // 不知道是不是因为el = Container的原因，在后退的时候会出现el = undefined的情况
    if (el) {
      this.elemRef = el.elemRef;
    }
  }

  /** 计算popup要偏移的top值 */
  private calcPopupTop(): number {
    // 防止在render时读取
    if (this.elemRef) {
      const offsetTop = this.elemRef.offsetTop;
      const height = this.elemRef.clientHeight;

      return height + offsetTop;
    }

    return 0;
  }

  private setSelected(index: number | undefined): void {
    this.setState({
      selected: index
    });
  }

  /** Popup关闭时触发 */
  private onClose(e: React.MouseEvent<HTMLDivElement>, visible: boolean): void {
    const { onClose } = this.props;

    if (visible === false) {
      this.setState({
        selected: undefined
      });

      if (onClose) {
        onClose();
      }
    }
  }

  /** 防止选择的时候关闭蒙层 */
  private preventDefault(e: React.MouseEvent<HTMLDivElement>): void {
    e.preventDefault();
    e.stopPropagation();
  }

  public render(): JSX.Element[] {
    const { className, style, activeLabelCls, activeChoiceCls, popupCls, iconCls, service, contentMaskCls } = this.props;
    const { selected } = this.state;
    const data = this.props.data;

    return [
      <Container
        key="Container"
        className={classNames(prefixCls, className)}
        style={style}
        alignItems="center"
        justifyContent="space-between"
        ref={this.saveElemRef}
      >
        {data.map((item, i) => (
          <Container
            className={classNames({
              [activeLabelCls || '']: selected === i
            })}
            key={item.label}
            alignItems="center"
            justifyContent="center"
            flex={1}
            onClick={() => this.setSelected(i)}
          >
            <IconText
              className={classNames({
                [`${prefixCls}-active`]: selected === i
              })}
              iconCls={`${prefixCls}-arrow`}
              textCls={`${prefixCls}-label`}
              src={selected === i ? upSvg : downSvg}
              text={item.label}
              flexDirection="row-reverse"
            />
          </Container>
        ))}
      </Container>,
      <Popup
        key="Popup"
        contentCls={classNames(`${prefixCls}-maskContent`, contentMaskCls)}
        visible={selected !== undefined}
        animation={false}
        onClose={this.onClose}
        closable
      >
        <Popup.Content>
          <div
            className={classNames(`${prefixCls}-popup`, popupCls)}
            style={{
              top: this.calcPopupTop()
            }}
            onClick={this.preventDefault}
          >
            {selected !== undefined && selected >= 0 ? (
              <Choice
                service={service && service[selected]}
                data={data[selected].options}
                iconCls={classNames(`${prefixCls}-choice-icon`, iconCls)}
                activeCls={classNames(`${prefixCls}-choice-checked`, activeChoiceCls)}
                itemCls={`${prefixCls}-choice-item`}
                textCls={`${prefixCls}-choice-text`}
              />
            ) : null}
          </div>
        </Popup.Content>
      </Popup>
    ];
  }
}

export default Filter;