import yj from '../../typings';
import * as React from 'react';
import * as classNames from 'classnames';
import StateMachine, { BaseProps as StateBaseProps, Type as StateType } from '../StateMachine';
import isHttp from '../../utils/isHttp';
import './index.scss';

export interface BaseProps {
  /** div模式不会自动设置高度，img模式会根据宽度设置高度 */
  mode?: 'div' | 'img';
  /** 是否要添加内阴影 */
  shadow?: boolean;
  /** 主机地址 */
  host?: string;
}

interface Props extends StateBaseProps, yj.ComponentProps, BaseProps {
  /** 正常状态想展示的图片，即img标签的src属性 */
  completeView: string;
  /** 加载失败的视图 */
  failView?: string;
  /** 加载中的视图 */
  loadingView?: string;
  /** 是否启用状态机 */
  stateMachine?: boolean;
  /** 点击事件 */
  onClick?: yj.OnClick<HTMLDivElement>;
  /** 图片加载完成或失败 */
  onLoad?: () => void;
}

interface State {
  state: StateType;
}

const prefixCls = 'yj-component-imgview';
const throttle: _.LoDashStatic['throttle'] = require('lodash/throttle');

class ImgView extends React.Component<Props, State> {
  constructor(props: Props, state: State) {
    super(props, state);

    this.onImgLoad = this.onImgLoad.bind(this);
    this.onImgError = this.onImgError.bind(this);
  }

  public static defaultProps: Props = {
    mode: 'div',
    completeView: '',
    failView: undefined,
    loadingView: undefined,
    stateMachine: true,
    onClick: () => null
  };

  public static defaultHost: string = '';

  public state: State = {
    state: 'loading'
  };

  /** 组件是否已经被卸载 */
  private isUnmount: boolean | undefined;
  /** 用于加载图片，改变ImgView的状态 */
  private imgObj: HTMLImageElement | undefined;
  /** Item的引用 */
  private itemRef: HTMLDivElement | undefined;
  /** 限制setImgSrc的频率 */
  private throttled: ((props: Props) => void) & _.Cancelable = throttle((props) => this.setImgSrc(props), 300, { leading: false });

  public shouldComponentUpdate(nextProps: Props, nextState: State): boolean {
    const stateChange = this.state.state !== nextState.state;

    return stateChange;
  }

  public componentDidMount(): void {
    this.imgObj = new Image();

    this.imgObj.addEventListener('load', this.onImgLoad);
    this.imgObj.addEventListener('error', this.onImgError);
    this.setImgSrc(this.props);
  }

  public componentWillUnmount(): void {
    this.isUnmount = true;

    if (this.imgObj) {
      this.imgObj.removeEventListener('load', this.onImgLoad);
      this.imgObj.removeEventListener('error', this.onImgError);
    }
  }

  public componentWillReceiveProps(nextProps: Props): void {
    if (this.props.completeView !== nextProps.completeView) {
      this.throttled(nextProps);
    }
  }

  private getSrc(props: Props, src: string | undefined): string {
    const host = props.host || ImgView.defaultHost;
    const url = src && isHttp(src) ? src : host + src;

    return url;
  }

  /** 图片的onload事件 */
  private onImgLoad(): void {
    if (!this.isUnmount) {
      this.setState({ state: 'complete' }, () => {
        if (this.props.onLoad) {
          this.props.onLoad();
        }
      });
    }
  }

  /** 图片的onerror事件 */
  private onImgError(): void {
    if (!this.isUnmount) {
      this.setState({ state: 'fail' }, () => {
        if (this.props.onLoad) {
          this.props.onLoad();
        }
      });
    }
  }

  /** 设置this.img的src属性 */
  private setImgSrc(props: Props): void {
    const src = this.getSrc(props, props.completeView);

    if (src && this.imgObj) {
      this.imgObj.src = src;
    } else {
      this.setState({
        state: 'fail'
      });
    }
  }

  private saveItemRef(el: HTMLDivElement | null): void {
    if (el) {
      this.itemRef = el;
    }
  }

  /** 计算图片高度，在mode='img'的情况下使用 */
  private calcHeight(): number {
    if (this.itemRef && this.imgObj) {
      const width = this.itemRef.clientWidth;

      if (this.imgObj.width) {
        return (this.imgObj.height / this.imgObj.width) * width;
      }
    }

    return 0;
  }

  public render(): JSX.Element {
    const { loadingStyle, mode, shadow, loadingView, completeView, failView, onClick, stateMachine, className } = this.props;
    const state = this.state.state;
    const complete = (
      <div
        className={classNames(`${prefixCls}-item`, `${prefixCls}-${mode}-mode`, `${prefixCls}-complete`, className)}
        style={{
          boxShadow: shadow ? '0 0 30px rgba(0, 0, 0, 0.9) inset' : undefined,
          backgroundImage: completeView ? `url(${this.getSrc(this.props, completeView)})` : 'none',
          height: mode === 'img' ? this.calcHeight() : undefined
        }}
        onClick={state === 'complete' ? onClick : undefined}
        ref={(el) => this.saveItemRef(el)}
      />
    );

    return stateMachine ? (
      <StateMachine
        state={this.state.state}
        loadingStyle={loadingStyle}
      >
        <StateMachine.Complete>{complete}</StateMachine.Complete>
        <StateMachine.Fail>
          <div
            className={classNames(`${prefixCls}-item`, `${prefixCls}-${mode}-mode`, `${prefixCls}-fail`, className)}
            style={{
              backgroundImage: failView ? `url(${this.getSrc(this.props, failView)})` : 'none'
            }}
          />
        </StateMachine.Fail>
        {loadingView || <StateMachine.Loading style={loadingStyle} />}
      </StateMachine>
    ) : complete;
  }
}

export default ImgView;