import yj from '../../typings';
import * as React from 'react';
import * as classNames from 'classnames';
import { observer } from 'mobx-react';
import Service from './service';
import Keyboard, { Props as KeyboardProps } from '../Keyboard';
import Popup from '../Popup';
import getNumberByComputed from '../../utils/getNumberByComputed';

export interface Base {
  service?: Service;
  disabled?: boolean;
  placeholder?: string;
  maxLength?: KeyboardProps['maxLength'];
}

interface Props extends yj.ComponentProps, Base {
  type: KeyboardProps['type'];
  confirmLabel?: KeyboardProps['confirmLabel'];
  value: string;
  onOk?: (value: string) => void;
}

interface State {
  /** 显示的值 */
  value: string;
  /** 是否处于焦点中 */
  visible: boolean;
  lineHeight: number | string | undefined;
}

const prefixCls = 'yj-component-input';
@observer class Custom extends React.Component<Props, State> {
  constructor(props: Props, state: State) {
    super(props, state);

    this.saveElemRef = this.saveElemRef.bind(this);
    this.onChange = this.onChange.bind(this);
    this.onConfirm = this.onConfirm.bind(this);
    this.onClose = this.onClose.bind(this);
    this.toggleVisible = this.toggleVisible.bind(this);
    this.onMaskClose = this.onMaskClose.bind(this);
  }

  public static defaultProps: Props = {
    value: '',
    disabled: false,
    type: 'number'
  };

  public state: State = {
    value: this.props.value,
    visible: false,
    lineHeight: undefined
  };

  private elemRef: HTMLDivElement | undefined | null;

  public componentDidMount(): void {
    this.setLineHeight();
  }

  public componentWillReceiveProps(nextProps: Props): void {
    if (this.props.value !== nextProps.value) {
      this.setState({
        value: nextProps.value
      });
    }
  }

  private saveElemRef(el: HTMLDivElement | null): void {
    this.elemRef = el;
  }

  private setLineHeight(): void {
    if (this.elemRef) {
      const lineHeight = getNumberByComputed(getComputedStyle(this.elemRef).lineHeight);
      const fontSize = getNumberByComputed(getComputedStyle(this.elemRef).fontSize);
      if (fontSize && !lineHeight) {
        this.setState({
          lineHeight: fontSize + 'px'
        });
      }
    }
  }

  private onChange(value: string): void {
    this.setState({ value });
  }

  private onConfirm(value: string): void {
    if (this.props.service) {
      this.props.service.changeValue(value);
    }

    if (this.props.onOk) {
      this.props.onOk(value);
    }

    this.toggleVisible();
  }

  private onClose(): void {
    const { service } = this.props;

    this.setState({
      value: service && service.getValue() || ''
    });
    this.toggleVisible();
  }

  private toggleVisible(): void {
    this.setState({
      visible: !this.state.visible,
    });
  }

  private onMaskClose(): void {
    if (this.props.service) {
      this.props.service.changeValue(this.state.value);
    }

    this.setState({
      visible: !this.state.visible,
    });
  }

  public render(): JSX.Element {
    const { placeholder, disabled, type, className, style, confirmLabel, maxLength } = this.props;
    const { visible } = this.state;
    const value = this.state.value;

    return (
      <Popup
        visible={visible}
        contentCls={`${prefixCls}-keyboard`}
        maskCls={`${prefixCls}-mask`}
        onClose={this.onMaskClose}
        closable
      >
        <Popup.Button className={className} disabled={disabled} onClick={this.toggleVisible}>
          <div
            className={prefixCls}
            style={{
              lineHeight: this.state.lineHeight,
              ...style
            }}
            ref={this.saveElemRef}
          >
            {!value && <div className={`${prefixCls}-placeholder`}>{placeholder}</div>}
            <div className={`${prefixCls}-custom`}>
              <div
                className={classNames({
                  [`${prefixCls}-custom-focus`]: visible,
                  [`${prefixCls}-disabled`]: disabled
                })}
                style={{
                  display: 'inline-block'
                }}
              >{value}</div>
            </div>
          </div>
        </Popup.Button>
        <Popup.Content>
          <Keyboard
            onChange={this.onChange}
            onConfirm={this.onConfirm}
            onClose={this.onClose}
            confirmLabel={confirmLabel}
            maxLength={maxLength}
            type={type}
            value={value || ''}
          />
        </Popup.Content>
      </Popup>
    );
  }
}

export default Custom;
