import * as React from 'react';
import { observer } from 'mobx-react';
import Service from '../service';
import MarkerService from '../service/marker';
import Toast from '../../Toast';
import Link from '../../Link';
import './index.scss';

export interface BaseProps {
}

interface Props extends BaseProps {
  service: Service;
  marker: BMap.Marker;
  markerService: MarkerService;
  title: string;
  address: string;
}

const prefixCls = 'yj-component-map-infowindow';
@observer class InfoWindow extends React.Component<Props> {
  constructor(props: Props) {
    super(props);

    this.onNavigation = this.onNavigation.bind(this);
  }

  private label: BMap.Label | undefined;

  public componentWillUnmount(): void {
    if (this.label) {
      this.label.removeEventListener('click', this.onNavigation);
    }
  }

  private create(props: Props = this.props, distance: string): void {
    const marker = props.marker;
    const service = props.service;
    const display = service.activeMarker === props.markerService ? 'block' : 'none';

    if (marker) {
      const getLabel = marker.getLabel();
      if (!getLabel) {
        this.label = new BMap.Label(this.getContent(props, distance));
        this.label.setStyle({
          border: 0,
          backgroundColor: 'transparent',
          left: '50%',
          transform: 'translate(-webkit-calc(-50% + 16px), -webkit-calc(-100% - 11px))',
          display
        });
        this.label.addEventListener('click', this.onNavigation);
        marker.setLabel(this.label);
      } else {
        getLabel.setContent(this.getContent(props, distance));
        getLabel.setStyle({ display });
      }
    }
  }

  private onNavigation(e: { domEvent: MouseEvent }): void {
    e.domEvent.preventDefault();
    e.domEvent.stopPropagation();
    const target = e.domEvent.target as HTMLDivElement;
    const isNavigationTarget = target.dataset.type === 'navigation';

    if (isNavigationTarget) {
      const { service } = this.props;
      const isOrgin = service.activeMarker === service.center;
      const geocoderResult = service.geolocation.getValue();
      const navigationDisabled = isOrgin && !geocoderResult;

      if (navigationDisabled || !service.activeMarker) {
        Toast.show('正在定位中...');
      } else if (service.center.mapPoint && geocoderResult) {
        const originLat = isOrgin ? geocoderResult.point.lat : service.center.mapPoint.lat;
        const originLng = isOrgin ? geocoderResult.point.lng : service.center.mapPoint.lng;
        const originName = isOrgin ? '当前位置' : service.center.title;
        const activeMarker = service.activeMarker as MarkerService;

        if (activeMarker.mapPoint) {
          Link.go(`https://api.map.baidu.com/direction?origin=latlng:${originLat},${originLng}|name:${originName}&
          destination=name:${activeMarker.title}|latlng:${activeMarker.mapPoint.lat},${activeMarker.mapPoint.lng}&
          mode=driving&region=${geocoderResult.addressComponents.city}&output=html&src=锐云|悦家`);
        } else {
          console.warn('component->Map->widget->InfoWindow->index->onNavigation', 'activeMarker.point未定义');
        }
      } else {
        console.warn('component->Map->widget->InfoWindow->index->onNavigation', 'model.origin.point未定义');
      }
    }
  }

  private getContent(props: Props = this.props, distance: string): string {
    const { title, address, service } = props;
    const isOrgin = service.activeMarker === service.center;
    const navigationDisabled = isOrgin && !service.geolocation.getValue();

    const distanceRender = distance ? `
      <div class="${prefixCls}-content">
        距离约${distance}
      </div>
    ` : '';
    return `
      <div class="${prefixCls} ${navigationDisabled ? prefixCls + '-navigationDisabled' : ''}">
        <div class="${prefixCls}-label">
          <div class="${prefixCls}-title">${title}</div>
          <div class="${prefixCls}-content">
            ${address}
          </div>
          ${distanceRender}
        </div>
        <div class="${prefixCls}-navigation" data-type="navigation">导航</div>
        <svg class="${prefixCls}-triangle-wrap">
          <polygon
            class="${prefixCls}-triangle"
            points="${`0,0 ${10},${9} ${20},0`}"
          ></polygon>
        </svg>
      </div>
    `;
  }

  public render(): null {
    const { markerService, service } = this.props;
    const distance = service.activeMarker !== service.center ? markerService.distance : '';
    this.create(this.props, distance);
    return null;
  }
}

export default InfoWindow;