import yj from '../../typings';
import * as React from 'react';
import { observer } from 'mobx-react';
import Item, { Base as ItemBase , Data, itemHeight, num, Value } from './Item';
import './index.scss';

export interface Base extends ItemBase {
  /** 在onScollEnd后触发，用于判断哪些列需要保持原来的选中索引，哪些列需要初始化为0 */
  formatSelectedCol?: (currentCol: number, selectedItem: SelectedItem[]) => number[];
}

export interface Props extends yj.ComponentProps, Base {
  /** 当是级联的时候是一维数组，否则是对象二维数组 */
  data: Data[] | Data[][];
  value: Value[] | undefined;
  onScrollEnd: (currentCol: number, currentValue: Value, selectedIndex: number, allValue: Value[]) => void;
  /** 组件创建成功后执行 */
  onCreate?: (data: Data[][]) => void;
}

export type SelectedItem = (number | undefined);

interface State {
  /** 用于滚动后刷新render */
  refreshId: number;
}

const prefixCls = 'yj-component-picker';

@observer class Picker extends React.Component<Props, State> {
  constructor(props: Props) {
    super(props);

    if (this.props.value) {
      for (let i = 0; i < this.props.value.length; i++) {
        this.selectedIndex[i] = this.props.value[i].index;
      }
    }

    // 初始化数据
    this.switchData(this.props, 0);
  }

  public state: State = {
    refreshId: 0
  };
  /** 滚动条选择的项 */
  private selectedIndex: SelectedItem[] = [];
  private data: Data[][] = [];

  public componentDidMount(): void {
    if (this.props.onCreate) {
      this.props.onCreate(this.data);
    }
  }

  public componentWillReceiveProps(nextProps: Props): void {
    if (this.props.data !== nextProps.data) {
      this.switchData(nextProps, 0);
      this.setState({
        refreshId: this.state.refreshId + 1
      });
    }
  }

  /** 根据是否是级联，来确定是否要格式化数据 */
  private switchData(props: Props, startCol: number): void {
    if (props.cascade) {
      const propsData = props.data as Data[];
      this.formatData(propsData, startCol);
    } else {
      this.data = props.data as Data[][];
    }
  }

  /** 获取被选中的列的子项选中的索引 */
  private getSelectedColIndex(col: number): number {
    const selectedIndex = this.selectedIndex[col];
    return selectedIndex ? selectedIndex : 0;
  }

  /** 设置格式化后的数据 */
  private formatData(data: Data[], startCol: number = 0): void {
    const each = (eachData: Data[]) => {
      // 判断数据是否为空
      if (eachData === undefined || (eachData.length === 0)) {
        return;
      } else {
        const selectedIndex = this.getSelectedColIndex(startCol);
        let selectedData = eachData[selectedIndex] as Data | undefined;
        // 如果selectedData不存在就取索引为0的数据
        if (selectedData === undefined) {
          this.selectedIndex[startCol] = 0;
          selectedData = eachData[0];
        }
        const children = selectedData.children;
        this.data[startCol] = [];

        for (let i = 0; i < eachData.length; i++) {
          this.data[startCol].push(eachData[i]);
        }
        startCol++;
        each(children as Data[]);
      }
    };

    // 如果是第一列就先初始化
    if (startCol === 0) {
      this.data[0] = [];
      for (let i = 0; i < data.length; i++) {
        const dataitem = data[i];
        const selectedIndex = this.getSelectedColIndex(0);
        this.data[0].push(dataitem);

        // 找到第一列被选中的项，把他的children拿去递归
        if (selectedIndex === i && dataitem.children) {
          startCol++;
          each(dataitem.children);
        }
      }
    } else {
      each(this.data[startCol]);
    }
  }

  /** 滚动结束后 */
  private onScrollEnd(currentCol: number, currentValue: Value, selectedIndex: number): void {
    const { formatSelectedCol, cascade, onScrollEnd } = this.props;
    const allValue = [];
    // 先更新到最新的值
    this.selectedIndex[currentCol] = selectedIndex;

    if (formatSelectedCol) {
      this.selectedIndex = formatSelectedCol(currentCol, this.selectedIndex);
    } else if (cascade) {
      // 将currentCol后续的列初始化为0，重置子项列表的选中项
      for (let i = currentCol + 1; i < this.data.length; i++) {
        this.selectedIndex[i] = 0;
      }
    }

    this.switchData(this.props, currentCol);

    for (let i = 0; i < this.data.length; i++) {
      const index = this.selectedIndex[i] || 0;
      const item = this.data[i][index];
      allValue.push({
        label: item.label,
        value: item.value,
        index
      });
    }

    onScrollEnd(currentCol, currentValue, selectedIndex, allValue);
    this.setState({
      refreshId: this.state.refreshId + 1
    });
  }

  public render(): JSX.Element {
    const { className, id, style, itemCls, textCls, cascade, value } = this.props;

    return (
      <div
        className={className}
        id={id}
        style={style}
      >
        <div className={`${prefixCls}-panel`}>
          <div className={`${prefixCls}-mask`} style={{ backgroundSize: `100% ${itemHeight * num}px` }} />
          <div className={`${prefixCls}-indicator`} style={{ top: `${itemHeight * num}px` }} />
          <div className={`${prefixCls}-content`}>
            {this.data.map((item, col) => (
              <Item
                key={col}
                data={item}
                value={value ? value[col] : item[col]}
                itemCls={itemCls}
                textCls={textCls}
                cascade={cascade}
                onScrollEnd={(currentValue, index) => this.onScrollEnd(col, currentValue, index)}
                selectedIndex={this.getSelectedColIndex(col)}
              />
            ))}
          </div>
        </div>
      </div>
    );
  }
}

export default Picker;