import * as React from 'react';
import * as classNames from 'classnames';
import { observer } from 'mobx-react';
import PickerPanel, { Base as PickerPanelBase, Props as PickerPanelProps } from './Picker';
import { Value, Data } from './Item';
import Popup from '../Popup';
import IconText from '../IconText';
import Service from './service';
import svgRootPaht from '../../utils/svgRootPath';
const rightSvg = svgRootPaht + require('../../assets/right.svg').id;

export interface Base extends PickerPanelBase {
  /** 点击弹出Picker的按钮的样式 */
  buttonCls?: string;
  /** 是否禁用 */
  disabled?: boolean;
  /** 取消的文本 */
  dismissText?: string;
  /** 确定的文本 */
  okText?: string;
  /** 取消和确定之间的标题 */
  title?: string;
  /** 占位字符 */
  placeholder?: string;
  /** 是否在按钮上显示向右的箭头 */
  rightIcon?: boolean;
}

export interface Props extends Base {
  service?: Service;
  data: PickerPanelProps['data'];
  /** 格式化显示的文本 */
  format?: (value: Value[]) => string;
  /** 确定的回调 */
  onOk?: (value: Value[]) => void;
}

interface State {
  visible: boolean;
}

const prefixCls = 'yj-component-picker';
@observer class Picker extends React.Component<Props, State> {
  constructor(props: Props, state: State) {
    super(props, state);

    this.toogleVisible = this.toogleVisible.bind(this);
    this.onScrollEnd = this.onScrollEnd.bind(this);
    this.onCreate = this.onCreate.bind(this);
    this.onOk = this.onOk.bind(this);
  }

  public static defaultProps: Props = {
    data: [],
    dismissText: '取消',
    okText: '确定',
    placeholder: '请选择',
    format: (value) => value.map((item) => item.label).join(',')
  };

  public state: State = {
    visible: false
  };

  private value: Value[] = [];

  public componentDidMount(): void {
    const { data, cascade } = this.props;

    if (cascade && typeof (data as Data[][])[0][0] !== 'undefined') {
      console.error('级联模式下data只能是一维数组', data);
    }

    if (!cascade && typeof (data as Data[][])[0][0] === 'undefined') {
      console.error('非级联模式下data只能是二维数组', data);
    }
  }

  public componentWillReceiveProps(nextProps: Props): void {
    if (!nextProps.cascade) {
      this.setValueWhenOverDataLength(nextProps);
    }
  }

  private toogleVisible(): void {
    this.setState({
      visible: !this.state.visible
    });
  }

  private onOk(): void {
    if (this.props.service) {
      this.props.service.changeValue(this.value);
    }

    if (this.props.onOk) {
      this.props.onOk(this.value);
    }

    this.toogleVisible();
  }

  /** 索引超过数据长度时使用，仅用于非级联的模式 */
  private setValueWhenOverDataLength(props: Props): void {
    // 如果索引超出数据的长度，就取数据最后一位
    const data = props.data as Data[][];
    const value = props.service && props.service.getValue();
    const newAllValue: Value[] = [];

    if (!value) {
      return;
    }

    for (let i = 0; i < data.length; i++) {
      // 有可能value[i]不存在
      if (value[i]) {
        if (value[i].index >= data[i].length) {
          const last = data[i].length - 1;
          const newValue = data[i][last];
          newAllValue[i] = {
            label: newValue.label,
            value: newValue.value,
            index: last
          };
        }
      }
    }

    // 当新值和旧值不一致时才去触发changeValue
    if (props.service && newAllValue.length && JSON.stringify(newAllValue.slice()) !== JSON.stringify(this.value.slice())) {
      this.value = newAllValue;
      props.service.changeValue(this.value);
    }
  }

  private onScrollEnd(currentCol: number, currentValue: Value, selectedIndex: number, allValue: Value[]): void {
    this.value = allValue;
  }

  private onCreate(data: Data[][]): void {
    const serviceValue = this.props.service && this.props.service.getValue();
    if (serviceValue) {
      this.value = serviceValue;
    } else {
      for (let i = 0; i < data.length; i++) {
        const item = data[i][0];
        this.value[i] = {
          label: item.label,
          value: item.value,
          index: 0
        };
      }
    }
  }

  public render(): JSX.Element {
    const { buttonCls, disabled, dismissText, okText, title, service, placeholder, rightIcon } = this.props;
    const format = this.props.format as ((value: Value[]) => string);
    const value = service && service.getValue();
    const label = value ? format(value) : placeholder as string;

    return (
      <Popup
        visible={this.state.visible}
        contentCls={`${prefixCls}-popup`}
      >
        <Popup.Button
          className={classNames(buttonCls, {
            [`${prefixCls}-placeholder`]: !value
          })}
          onClick={this.toogleVisible}
          disabled={disabled}
        >
          {rightIcon ? (
            <IconText
              iconCls={`${prefixCls}-icontext-icon`}
              src={rightSvg}
              text={label}
              flexDirection="row-reverse"
            />
          ) : label}
        </Popup.Button>
        <Popup.Content>
          <div className={`${prefixCls}-popup-header`}>
            <div className={`${prefixCls}-popup-button ${prefixCls}-popup-header-left`} onClick={this.toogleVisible}>
              {dismissText}
            </div>
            {title ? <div className={`${prefixCls}-popup-button ${prefixCls}-popup-title`}>{title}</div> : null}
            <div className={`${prefixCls}-popup-button ${prefixCls}-popup-header-right`} onClick={this.onOk}>
              {okText}
            </div>
          </div>
          <PickerPanel
            {...this.props}
            data={this.props.data}
            value={service && service.getValue()}
            onScrollEnd={this.onScrollEnd}
            onCreate={this.onCreate}
          />
        </Popup.Content>
      </Popup>
    );
  }
}

export default Picker;