import yj from '../../typings/index';
import * as React from 'react';
import * as classNames from 'classnames';
import Mask, { Props as MaskProps } from '../Mask';
import { Props as DialogProps } from '../Mask/Dialog';
import Button from './Button';
import Content from './Content';
import './index.scss';

interface Props extends DialogProps, MaskProps {
  /** 是否开启动画 */
  animation?: boolean;
  refreshId?: any;
}

interface State {
  /** 关闭中 */
  closeing: boolean;
  buttonView: any;
  contentView: any;
}

export interface View {
  className?: string;
  onClick: yj.OnClick<HTMLDivElement>;
  disabled?: boolean;
}

interface ViewType {
  displayName: string;
}

const prefixCls = 'yj-component-popup';
class Popup extends React.Component<Props, State> {
  constructor(props: Props, state: State) {
    super(props, state);

    this.onClose = this.onClose.bind(this);
    this.onClick = this.onClick.bind(this);
    this.lazyClose = this.lazyClose.bind(this);
  }

  public static defaultProps: Props = {
    animation: true,
    visible: false,
    children: null,
    onClose: () => null
  };

  public state: State = {
    closeing: false,
    buttonView: undefined,
    contentView: undefined,
  };

  /** 触发Popup显示的按钮 */
  public static Button: typeof Button = Button;
  /** 装载Popup的内容 */
  public static Content: typeof Content = Content;
  private timer: NodeJS.Timer | number | undefined;
  /** 是否处于动画执行的状态中 */
  private animation: boolean = false;

  public componentDidMount(): void {
    this.setView();
  }

  public componentWillReceiveProps(nextProps: Props): void {
    if (this.props.children !== nextProps.children || this.props.refreshId !== nextProps.refreshId) {
      this.setView(nextProps);
    }

    if (this.props.visible !== nextProps.visible) {
      this.onAnimation(nextProps.visible || false);
    }
  }

  public componentWillUnmount(): void {
    if (this.timer) {
      clearTimeout(this.timer as number);
    }
  }

  /** 动画进行中 */
  private onAnimation(visible: boolean, callback?: () => void): void {
    if (this.animation) {
      return;
    }

    this.animation = true;
    if (visible === false) {
      this.setState({
        closeing: true
      }, () => {
        this.lazyClose(callback);
        this.animation = false;
      });
    } else {
      this.setState({
        closeing: false
      });
      if (callback) {
        callback();
      }
      this.timer = setTimeout(() => {
        this.animation = false;
      }, 300);
    }
  }

  /** 延迟关闭 */
  private lazyClose(callback?: () => void): void {
    if (this.props.animation) {
      this.timer = setTimeout(() => {
        this.setState({
          closeing: false
        });

        if (callback) {
          callback();
        }
      }, 300);
    } else {
      this.setState({
        closeing: false
      });

      if (callback) {
        callback();
      }
    }
  }

  private onClose(e: React.MouseEvent<HTMLDivElement>): void {
    this.onAnimation(false, () => {
      if (this.props.onClose) {
        this.props.onClose(e, false);
      }
    });
  }

  private onClick(e: React.MouseEvent<HTMLDivElement>, onClick: (e: React.MouseEvent<HTMLDivElement>) => void): void {
    this.onAnimation(true, () => onClick(e));
  }

  private setView(props: Props = this.props): void {
    let buttonView;
    let contentView;

    React.Children.forEach(props.children, (child) => {
      const view = child as React.ReactElement<View>;
      const type = view.type as ViewType;
      if (type.displayName === 'PopupButton') {
        buttonView = React.cloneElement(view as any, {
          key: 'PopupButton',
          prefixCls,
          onClick: (e: React.MouseEvent<HTMLDivElement>) => this.onClick(e, view.props.onClick)
        });
      }

      if (type.displayName === 'PopupContent') {
        contentView = child;
      }
    });

    this.setState({
      buttonView, contentView
    });
  }

  public render(): JSX.Element[] {
    const { animation, contentCls, maskCls, visible } = this.props;
    const hidden = (visible === false && this.state.closeing === false && this.animation === false) ? false : true;

    return [
      this.state.buttonView,
      <Mask
        key="PopupMask"
        {...this.props}
        visible={visible || hidden}
        maskCls={classNames(maskCls, {
          [`${prefixCls}-fadein`]: animation,
          [`${prefixCls}-fadeout`]: animation && this.state.closeing
        })}
        contentCls={classNames(contentCls, {
          [`${prefixCls}-enter`]: animation,
          [`${prefixCls}-leave`]: animation && this.state.closeing
        })}
        onClose={this.onClose}
      >
        {this.state.contentView}
      </Mask>
    ];
  }
}

export default Popup;
