import yj from '../../typings';
import * as React from 'react';
import Loading from './Loading';
import Complete from './Complete';
import Fail from './Fail';
import Empty from './Empty';
const lowerFirst: _.LoDashStatic['lowerFirst'] = require('lodash/lowerFirst');
const throttle: _.LoDashStatic['throttle'] = require('lodash/throttle');

/** 状态类型 */
export type Type = 'complete' | 'empty' | 'fail'  | 'loading'| 'timeout' | undefined;

export interface BaseProps {
  loadingStyle?: React.CSSProperties;
}

interface Props extends BaseProps {
  /** 当前的状态类型 */
  state: Type;
  children?: any;
}

interface State {
  /** 渲染的状态视图 */
  render: JSX.Element | null;
}

/** 有限状态机 */
class StateMachine extends React.PureComponent<Props, State> {
  constructor(props: Props, state: State) {
    super(props, state);
    this.stateRender = this.stateRender.bind(this);
  }

  public state: State = {
    render: null,
  };

  /** 成功的视图 */
  public static Complete: typeof Complete = Complete;
  /** 失败的视图 */
  public static Fail: typeof Fail = Fail;
  /** 加载中的视图 */
  public static Loading: typeof Loading = Loading;
  /** 数据为空的视图 */
  public static Empty: typeof Empty = Empty;
  /** 限制stateRender的频率 */
  private throttled: ((props: Props) => void) & _.Cancelable = throttle((props) => this.stateRender(props), 100, { leading: false });

  public componentWillUnmount(): void {
    if (this.throttled) {
      this.throttled.cancel();
    }
  }

  public componentDidMount(): void {
    this.throttled(this.props);
  }

  public componentWillReceiveProps(nextProps: Props): void {
    const stateChange = this.props.state !== nextProps.state;
    const childrenChange = this.props.children !== nextProps.children;

    if (stateChange || childrenChange) {
      this.throttled(nextProps);
    }
  }

  /** 根据状态渲染视图 */
  public stateRender(props: Props): void {
    let render = null;

    // 根据当前状态赋予对应的视图
    React.Children.forEach(props.children, (child) => {
      const childElement = child as React.ReactElement<HTMLDivElement>;
      if (lowerFirst(childElement && childElement.type && (childElement.type as yj.ChildType).displayName) === props.state) {
        render = childElement;
      }
    });

    // 如果children里没有声明Loading视图就使用默认的
    if (!render && props.state === 'loading') {
      render = <StateMachine.Loading style={this.props.loadingStyle} />;
    }

    this.setState({
      render,
    });
  }

  public render(): JSX.Element | null {
    return this.state.render;
  }
}

export default StateMachine;