import * as React from 'react';
import { observer } from 'mobx-react';
import Request, { Params } from '../../app/request';
import StateMachine, { BaseProps as StateBaseProps } from '../StateMachine';
import ObListMap from '../../app/datatype/obListMap';
import ObDetailMap from '../../app/datatype/obDetailMap';
// import BaseService from '../../_base/service/base';
const isEmpty = require('lodash/isEmpty');

interface Props extends StateBaseProps {
  /** yj-framework中的Request对象 */
  request?: Request;
  /** 请求参数 */
  params?: Params;
  /** 延迟的加载时间 */
  delay?: number;
  /** 只有是Map对象时，才能够去使用缓存，并且键值应为请求的参数 */
  cache?: ObListMap<any, any> | ObDetailMap<any, any>;
  /** 用于是否重新请求 */
  refreshId?: any;
  /** 用于是否重新渲染 */
  renderId?: any;
}

@observer class View extends React.Component<Props> {
  constructor(props: Props) {
    super(props);
  }

  /** setTimeout对象 */
  private timer: NodeJS.Timer | number | undefined;
  /** 成功的视图 */
  public static Complete: typeof StateMachine.Complete = StateMachine.Complete;
  /** 失败的视图 */
  public static Fail: typeof StateMachine.Fail = StateMachine.Fail;
  /** 加载中的视图 */
  public static Loading: typeof StateMachine.Loading = StateMachine.Loading;
  /** 数据为空的视图 */
  public static Empty: typeof StateMachine.Empty = StateMachine.Empty;

  public shouldComponentUpdate(nextProps: Props): boolean {
    const renderId = this.props.renderId;
    const nextRenderId = nextProps.renderId;

    return (renderId !== undefined || nextRenderId !== undefined) ? renderId !== nextRenderId : true;
  }

  public componentDidMount(): void {
    if (this.props.request) {
      const cache = this.readCache(this.props.cache, this.props.params);

      if (cache) {
        this.props.request.setState('complete');
      } else {
        this.request(this.props, false);
      }
    } else {
      console.warn('没有设置request,只能应用complete状态');
    }
  }

  public componentWillReceiveProps(nextProps: Props): void {
    const paramsStr = typeof this.props.params === 'object' ? JSON.stringify(this.props.params) : this.props.params;
    const nextParamsStr = typeof nextProps.params === 'object' ? JSON.stringify(nextProps.params) : nextProps.params;
    const refreshIdChange = this.props.refreshId !== nextProps.refreshId;

    if (paramsStr !== nextParamsStr || refreshIdChange) {
      if (nextProps.request) {
        this.request(nextProps, true);
      }
    }
  }

  public componentWillUnmount(): void {
    if (this.timer) {
      clearTimeout(this.timer as NodeJS.Timer);
    }
    if (this.props.request && this.props.request.cancel) {
      this.props.request.cancel('cancel');
    }
  }

  /** 发送请求 */
  private request(props: Props, refreshChange: boolean): void {
    if (props.request) {
      const request = props.request;
      const delay = props.delay !== undefined ? props.delay : 300;

      if (delay) {
        if (refreshChange === false) {
          request.setState(undefined);
        }
        this.timer = setTimeout(() => {
          if (refreshChange === false) {
            request.setState('loading');
          }
          request.send(props.params).catch((err) => null);
        }, delay);
      } else {
        if (refreshChange === false) {
          request.setState('loading');
        }
        request.send(props.params).catch((err) => null);
      }
    }
  }

  /** 读取缓存 */
  private readCache(cache: Props['cache'], params?: Params): any | undefined {
    if (cache && params) {
      const data = cache.get(JSON.stringify(params));

      // 判断数据是否为空
      if (data && isEmpty(data) === false) {
        return data;
      }
    }
  }

  public render(): JSX.Element {
    return (
      <StateMachine
        state={this.props.request ? this.props.request.getState() : 'complete'}
        loadingStyle={this.props.loadingStyle}
      >
        {this.props.children}
      </StateMachine>
    );
  }
}

export default View;