import { observable, action } from 'mobx';
import { Props as RequestOptions } from '../../app/request';
import stringToHex from '../../utils/stringToHex';
import global from '../../app/global';
import Input from '../../component/Input/service';
// import NumberInput from '../../component/Input/service/number';
// import TextInput from '../../component/Input/service/text';
import Form from '../../component/Form/service';
import GetRequest from './get';
import PostRequest from './post';
import VerificationRequest from './verification';

type RequestType = 'register' | 'signin' | 'reset' | 'save';

interface Options {
  /** 用户缓存id */
  readonly userCacheId: string;
  /** localStorage中设置的key，用于存储用户信息，如果应用有修改用户信息的功能建议不配置该项 */
  readonly storageKey: string | undefined;
  /** 登录页面的域名 */
  readonly loginHost: string | undefined;
  /** 登录成功后的回调页面 */
  readonly loginCallBack: string | undefined;
}

interface State {
  /** 登录状态 */
  loginState: boolean | undefined;
  /** 关注状态 */
  subscribeState: boolean | undefined;
}

export interface Request {
  /** 获取用户信息的动作配置 */
  getRequest?: RequestOptions;
  /** 用户发送验证码的动作配置  */
  verificationRequest?: RequestOptions;
  /** 注册的请求 */
  registerRequest?: ExtendsRequestProps;
  /** 登录的请求 */
  signinRequest?: ExtendsRequestProps;
  /** 重置密码的请求 */
  resetRequest?: ExtendsRequestProps;
  /** 保存的请求 */
  saveRequest?: ExtendsRequestProps;
}

export interface ExtendsRequestProps extends RequestOptions {
  /** 用于初始化Form的componentState属性 */
  componentState?: boolean;
}

export interface Data {
  /** 微信的openId */
  openId: string;
  /** 系统的memeberId */
  memberId: string;
}

export interface StorageData extends Data {
  userCacheId: string;
}

class Account {
  constructor(options: Options & State, request: Request) {
    (['register', 'signin', 'reset', 'save'] as RequestType[]).forEach((requestName) => {
      const reqiestKey = requestName + 'Request';
      const formKey = requestName + 'Form';
      const propRequest = (request as any)[reqiestKey] as ExtendsRequestProps;

      if (propRequest) {
        (this as any)[reqiestKey] = new PostRequest(this, propRequest);
        (this as any)[formKey] = new Form({
          request: (this as any)[reqiestKey],
          complete: propRequest.componentState
        });
      }
    });

    if (request.getRequest) {
      this.getRequest = new GetRequest(this, request.getRequest);
    }

    if (request.verificationRequest) {
      this.verificationRequest = new VerificationRequest(this, request.verificationRequest);
    }

    /** 注册账号 */
    this.tel = new Input({
      fieldKey: 'tel',
      fieldName: '注册账号',
      required: true,
      validate: 'tel',
      // onChange: this.verificationRequest && this.verificationRequest.setCompelte,
      form: () => this.form
    });

    this.options = options;
    Account._loginState = options.loginState;
    Account._subscribeState = options.subscribeState;
    Account.loginAuth = this.loginAuth;
  }

  /** 配置参数 */
  public options!: Options;
  /** 获取用户信息的动作 */
  public getRequest: GetRequest | undefined;
  /** 注册的动作 */
  public registerRequest: PostRequest | undefined;
  /** 登录动作 */
  public signinRequest: PostRequest | undefined;
  /** 重置动作 */
  public resetRequest: PostRequest | undefined;
  /** 保存用户信息动作 */
  public saveRequest: PostRequest | undefined;
  public registerForm!: Form;
  public signinForm!: Form;
  public resetForm!: Form;
  public saveForm!: Form;
  /** 发送验证码动作 */
  public readonly verificationRequest: VerificationRequest | undefined;
  /** 从接口获取到用户数据 */
  @observable public data: Data | undefined;
  /** 用于选择提交表单的函数 */
  public _requestType: RequestType | undefined;
  /** 登录状态 */
  public static _loginState: boolean | undefined;
  /** 关注状态 */
  public static _subscribeState: boolean | undefined;
  /** 登录认证, 已登录调用callback，否则就跳登录页 */
  public static loginAuth: (callback?: () => void) => void;

  /** 注册账号 */
  public tel!: Input;

  /** 验证码 */
  public verificationCode: Input = new Input({
    fieldKey: 'verificationCode',
    fieldName: '验证码',
    required: true,
    // maxLength: 4,
    // type: 'integer',
    validate: 'integer',
    form: () => this.form
  });

  /** 密码 */
  public password: Input = new Input({
    fieldKey: 'password',
    fieldName: '密码',
    required: true,
    // type: 'password',
    // maxLength: 18,
    validate: (value: string | undefined) => {
      const regexp = new RegExp(/^(?![0-9]+$)(?![a-zA-Z]+$)[0-9A-Za-z]{8,16}$/);
      const isOrderly = regexp.test(value || '');

      if (this.password) {
        this.password.setPrompt(isOrderly ? '' : '密码格式为8-16位数字加字母');
      }

      return isOrderly;
    },
    form: () => this.form
  });

   /** 再次输入密码 */
  public repassword: Input = new Input({
    fieldKey: 'password',
    fieldName: '再次输入密码',
    required: true,
    // type: 'password',
    validate: (value: string | undefined) => {
      const isEqual = this.password.getValue() === value;

      if (this.repassword) {
        this.repassword.setPrompt(isEqual ? '' : '两次密码输入不一致');
      }

      return isEqual;
    },
    form: () => this.form
  });

  protected get form(): Form {
    // 默认登录signin
    switch (this._requestType) {
      case 'register':
        return this.registerForm;
      case 'reset':
        return this.resetForm;
      case 'save':
        return this.saveForm;
      default:
        return this.signinForm;
    }
  }

  /** 登录认证, 已登录调用callback，否则就跳公共登录页 */
  private loginAuth(callback?: () => void): void {
    if (Account._loginState === true && callback) {
      callback();
    } else {
      const options = this.options;

      if (options && options.loginHost && options.loginCallBack) {
        const { loginHost } = options;
        const loginCallBack = encodeURIComponent(options.loginCallBack);
        const pathname = stringToHex(encodeURIComponent(global.history.location.pathname));

        location.href = loginHost + '&callBackUrl=' + loginCallBack + '&route=' + pathname;
      } else {
        console.error('登录认证失败，缺少Account.loginHost或Account.loginCallBack');
      }
    }
  }

  @action public _setData(data: Data): void {
    const options = this.options;
    Account._loginState = true;

    const svaeData = {
      ...data,
      userCacheId: options && options.userCacheId
    };

    this.data = data;
    this.baseClear();

    if (options && options.storageKey) {
      window.localStorage.setItem(options.storageKey, JSON.stringify(svaeData));
    } else if (this.getRequest) {
      console.warn('Account.storageKey没有设置，数据不能存储在localStorage中');
    }
  }

  /** 清空基础字段的值 */
  @action public baseClear(): void {
    this.verificationCode.clear();
    this.tel.clear();
    this.password.clear();
    this.repassword.clear();

    // 有可能会不存在
    if (this.form) {
      this.form.setComplete(false);
    }
  }
}

export default Account;